@@ -422,7 +422,7 @@ def fixpos(sumstats,pos="POS",status="STATUS",remove=False, verbose=True,limit=2
         
         if sumstats[pos].dtype == "string" or sumstats[pos].dtype == "object":
             if verbose: log.write(' -Removing thousands separator "," or underbar "_" ...')
-            sumstats.loc[~is_pos_na, pos] = sumstats.loc[~is_pos_na, pos].str.replace(",|_", "")
+            sumstats.loc[~is_pos_na, pos] = sumstats.loc[~is_pos_na, pos].str.replace(",|_", "",regex=True)
 
         try:
             if verbose: log.write(' -Converting to Int64 data type ...')
@@ -1038,7 +1038,7 @@ def sortcoordinate(sumstats,chrom="CHR",pos="POS",reindex=True,verbose=True,log=
 ###############################################################################################################
 # 20221105
 def sortcolumn(sumstats,verbose=True,log=Log(),order = [
-        "SNPID","rsID", "CHR", "POS", "EA", "NEA", "EAF", "BETA", "SE", "Z",
+        "SNPID","rsID", "CHR", "POS", "EA", "NEA", "EAF", "MAF", "BETA", "SE", "Z",
         "CHISQ", "P", "MLOG10P", "OR", "OR_95L", "OR_95U", "INFO", "N","DIRECTION","STATUS"
            ]):
     if verbose: log.write("Start to reorder the columns...")
