@@ -5,9 +5,9 @@ import matplotlib.patches as patches
 import seaborn as sns
 import numpy as np
 import scipy as sp
-import gwaslab as gl
 from shutil import which
 from gwaslab.Log import Log
+from gwaslab.calculate_gc import lambdaGC
 from gwaslab.getsig import getsig
 from gwaslab.getsig import annogene
 from gwaslab.CommonData import get_chr_to_number
@@ -33,9 +33,15 @@ def mqqplot(insumstats,
           p=None,
           snpid=None,
           eaf=None,
+          chr_dict = get_chr_to_number(),
           vcf_path=None,
-          vcf_chr_dict=get_number_to_chr(),
-          gtf_path="defualt",
+          vcf_chr_dict = get_number_to_chr(),
+          gtf_path="default",
+          gtf_chr_dict = get_number_to_chr(),
+          gtf_gene_name=None,
+          rr_path="default",
+          rr_header_dict=None,
+          rr_chr_dict = get_number_to_chr(),
           mlog10p="MLOG10P",
           scaled=False,
           mode="mqq",
@@ -88,6 +94,7 @@ def mqqplot(insumstats,
           maf_bins=[(0, 0.01), (0.01, 0.05), (0.05, 0.25),(0.25,0.5)],
           maf_bin_colors = ["#f0ad4e","#5cb85c", "#5bc0de","#000042"],
           gc=True,
+          include_chrXYMT = True,
           title =None,
           mtitle=None,
           qtitle=None,
@@ -238,9 +245,11 @@ def mqqplot(insumstats,
     ## CHR & POS
     ## m, qq, b
     if "m" in mode or "r" in mode or "b" in mode: 
-        # CHR X,Y,MT conversion ############################
-        if sumstats[chrom].dtype =="str":
-            sumstats[chrom] = sumstats[chrom].map(get_chr_to_number(),na_action="ignore")
+        # convert CHR to int
+        ## CHR X,Y,MT conversion ############################
+        if pd.api.types.is_string_dtype(sumstats[chrom]) == True:
+        ## if chr is string dtype: convert using chr_dict
+            sumstats[chrom] = sumstats[chrom].map(chr_dict,na_action="ignore")
         ## CHR
         sumstats[chrom] = np.floor(pd.to_numeric(sumstats[chrom], errors='coerce')).astype('Int64')
         ## POS
@@ -346,7 +355,7 @@ def mqqplot(insumstats,
         sumstats = sumstats.loc[~(is_inf | is_na),:]
 
     # raw p for calculate lambda
-    p_toplot_raw = sumstats["scaled_P"].copy()
+    p_toplot_raw = sumstats[["CHR","scaled_P"]].copy()
     
     # filter out variants with -log10p < skip
     sumstats = sumstats.loc[sumstats["scaled_P"]>=skip,:]
@@ -542,7 +551,17 @@ def mqqplot(insumstats,
             ax4=ax1.twinx()
             most_left_snp = sumstats["i"].idxmin()
             rc_track_offset = sumstats.loc[most_left_snp,"i"]-sumstats.loc[most_left_snp,pos]
-            rc = get_recombination_rate(chrom=str(region[0]),build=build)
+            if rr_path=="default":
+                if rr_chr_dict is not None:
+                    rr_chr = rr_chr_dict[region[0]]
+                else:
+                    rr_chr = str(region[0])
+                rc = get_recombination_rate(chrom=rr_chr,build=build)
+            else:
+                rc = pd.read_csv(rr_path,sep="\t")
+                if rr_header_dict is not None:
+                    rc = rc.rename(columns=rr_header_dict)
+
             rc = rc.loc[(rc["Position(bp)"]<region[2]) & (rc["Position(bp)"]>region[1]),:]
             ax4.plot(rc_track_offset+rc["Position(bp)"],rc["Rate(cM/Mb)"],color="#5858FF",zorder=1)
             ax4.set_ylabel("Recombination rate(cM/Mb)")
@@ -563,8 +582,13 @@ def mqqplot(insumstats,
         if (gtf_path is not None ) and ("r" in mode):
             # load gtf
             if verbose: log.write(" -Loading gtf files from:" + gtf_path)
-            uniq_gene_region,exons = process_gtf (gtf_path = gtf_path ,region = region, region_flank_factor = region_flank_factor,build=build,region_protein_coding=region_protein_coding)
-            
+            uniq_gene_region,exons = process_gtf(gtf_path = gtf_path ,
+                                                    region = region, 
+                                                    region_flank_factor = region_flank_factor,
+                                                    build=build,
+                                                    region_protein_coding=region_protein_coding,
+                                                    gtf_chr_dict=gtf_chr_dict,
+                                                    gtf_gene_name=gtf_gene_name)
             n_uniq_stack = uniq_gene_region["stack"].nunique()
             stack_num_to_plot = max(taf[0],n_uniq_stack)
             ax3.set_ylim((-stack_num_to_plot*2-taf[1]*2,2+taf[1]*2))
@@ -576,7 +600,7 @@ def mqqplot(insumstats,
             linewidth_in_points=   pixels_per_track * pixels_per_point
             if verbose: log.write(" -plotting gene track..")
             
-            sig_gene_name = None
+            sig_gene_name = "Undefined"
             texts_to_adjust_left = []
             texts_to_adjust_middle = []
             texts_to_adjust_right = []
@@ -589,6 +613,8 @@ def mqqplot(insumstats,
                 if region_lead_grid is True and lead_snp_i > gene_track_start_i+row[3] and lead_snp_i < gene_track_start_i+row[4] :
                         gene_color="#FF0000"
                         sig_gene_name = row["name"]
+                        sig_gene_left = gene_track_start_i+row[3]
+                        sig_gene_right= gene_track_start_i+row[4]
                 else:
                     gene_color="#020080"
                 
@@ -612,7 +638,12 @@ def mqqplot(insumstats,
             
             # plot exons
             for index,row in exons.iterrows():
-                if (region_lead_grid is True) and row["name"]==sig_gene_name:
+                if not pd.isnull(row["name"]):
+                    if (region_lead_grid is True) and row["name"]==sig_gene_name:
+                        exon_color = region_lead_grid_line["color"]  
+                    else:
+                        exon_color="#020080" 
+                elif gene_track_start_i+row[3] > sig_gene_left and gene_track_start_i+row[4] < sig_gene_right:
                     exon_color = region_lead_grid_line["color"]  
                 else:
                     exon_color="#020080"
@@ -627,7 +658,7 @@ def mqqplot(insumstats,
         ax1.set_xticks(chrom_df.astype("float64"))
         ax1.set_xticklabels(chrom_df.index.map(get_number_to_chr()),fontsize=fontsize,family="sans-serif")
         
-        # regional plot - set X tick
+        ## regional plot - set X tick
         if region is not None:
             region_ticks = list(map('{:.3f}'.format,np.linspace(region[1], region[2], num=region_step).astype("int")/1000000)) 
             
@@ -667,7 +698,6 @@ def mqqplot(insumstats,
             ax1.set_ylim(bottom = skip)
 
 
-
 # Get top variants for annotation #######################################################
         if (anno and anno!=True) or (len(anno_set)>0):
             if len(anno_set)>0:
@@ -886,21 +916,30 @@ def mqqplot(insumstats,
     # ax2 qqplot
     if "qq" in mode:
         if verbose:log.write("Start to create QQ plot with "+str(len(sumstats))+" variants:")
+        # plotting qq plots using processed data after cut and skip
+        # select -log10 scaled p to plot
         p_toplot = sumstats["scaled_P"]
-            # select -log10 scaled p to plot
-            # sort x,y for qq plot
+        # min p value for uniform distribution 
         minit=1/len(p_toplot)
         if stratified is False:
+            # sort x,y for qq plot
+            # high to low
             observed = p_toplot.sort_values(ascending=False)
+            # uniform distribution using raw number -> -log10 -> observed number (omit variants with low -log10p)
             expected = -np.log10(np.linspace(minit,1,len(p_toplot_raw)))[:len(observed)]
             #p_toplot = sumstats["scaled_P"]
             ax2.scatter(expected,observed,s=8,color=colors[0])
         else:
             # stratified qq plot
             for i,(lower, upper) in enumerate(maf_bins):
+                # extract data for a maf_bin
                 databin = sumstats.loc[(sumstats["MAF"]>lower) &( sumstats["MAF"]<=upper),["MAF","scaled_P"]]
+                # raw data : varaints with maf(eaf_raw) in maf_bin
                 databin_raw = eaf_raw[(eaf_raw>lower) & (eaf_raw<=upper)]
+                # sort x,y for qq plot
+                # high to low
                 observed = databin["scaled_P"].sort_values(ascending=False)
+                # uniform distribution using raw number -> -log10 -> observed number (omit variants with low -log10p)
                 expected = -np.log10(np.linspace(minit,1,max(len(databin_raw),len(databin))))[:len(observed)]
                 label ="("+str(lower)+","+str(upper) +"]"
                 ax2.scatter(expected,observed,s=8,color=maf_bin_colors[i],label=label)
@@ -916,11 +955,14 @@ def mqqplot(insumstats,
         
         # calculate genomic inflation factor and add annotation
         if gc:
-            
-            observedMedianChi2 = sp.stats.chi2.isf( np.median(np.power(10,-p_toplot_raw)) ,1)
-            expectedMedianChi2 = sp.stats.chi2.ppf(0.5,1)
-            lambdagc=observedMedianChi2/expectedMedianChi2
-            if verbose: log.write(" -Calculating GC using P :",lambdagc)
+            #gc was calculated using raw data (before cut and skip)
+            p_toplot_raw = p_toplot_raw.rename(columns={"scaled_P":"MLOG10P"})
+            if verbose and not include_chrXYMT : log.write(" -Excluding chrX,Y, MT from calculation of lambda GC.")
+            lambdagc = lambdaGC(p_toplot_raw, mode="MLOG10P", include_chrXYMT=include_chrXYMT,log=log,verbose=False)
+            #observedMedianChi2 = sp.stats.chi2.isf( np.median(np.power(10,-p_toplot_raw)) ,1)
+            #expectedMedianChi2 = sp.stats.chi2.ppf(0.5,1)
+            #lambdagc=observedMedianChi2/expectedMedianChi2
+            if verbose: log.write(" -Calculating lambda GC:",lambdagc)
             ax2.text(0.10, 1.03,"$\\lambda_{GC}$ = "+"{:.4f}".format(lambdagc),
                      horizontalalignment='left',
                      verticalalignment='top',
@@ -978,24 +1020,33 @@ def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_thresh
     ref_genotype = read_vcf(vcf_path,region=vcf_chr_dict[region[0]]+":"+str(region[1])+"-"+str(region[2]),tabix=tabix)
     
     if verbose: log.write(" -Retrieving index...")
+    if verbose: log.write(" -Ref variants in the region: {}".format(len(ref_genotype["variants/POS"])))
     # match sumstats pos and ref pos: 
     # get ref index for its first appearance of sumstats pos
     sumstats["REFINDEX"] = sumstats[pos].apply(lambda x: np.where(ref_genotype["variants/POS"] == x )[0][0] if np.any(ref_genotype["variants/POS"] == x) else None)
-    
     # get lead variant id and pos
     lead_id = sumstats["scaled_P"].idxmax()
     lead_pos = sumstats.loc[lead_id,pos]
-
+    
+    # if lead pos is available: 
     if lead_pos in ref_genotype["variants/POS"]:
-        # if lead pos is available: 
+        
         # get ref index for lead snp
         lead_snp_ref_index = np.where(ref_genotype["variants/POS"] == lead_pos)[0][0]
-        #print(lead_snp_ref_index)
-        other_snps_ref_index = sumstats["REFINDEX"].values
+
+        # non-na other snp index
+        other_snps_ref_index = sumstats["REFINDEX"].dropna().astype("int").values
+        # get genotype 
         lead_snp_genotype = GenotypeArray([ref_genotype["calldata/GT"][lead_snp_ref_index]]).to_n_alt()
         other_snp_genotype = GenotypeArray(ref_genotype["calldata/GT"][other_snps_ref_index]).to_n_alt()
+        
         if verbose: log.write(" -Calculating Rsq...")
-        sumstats.loc[~sumstats["REFINDEX"].isna(),"RSQ"] = np.power(rogers_huff_r_between(lead_snp_genotype,other_snp_genotype)[0],2)
+        
+        if len(other_snp_genotype)>1:
+            valid_r2= np.power(rogers_huff_r_between(lead_snp_genotype,other_snp_genotype)[0],2)
+        else:
+            valid_r2= np.power(rogers_huff_r_between(lead_snp_genotype,other_snp_genotype),2)
+        sumstats.loc[~sumstats["REFINDEX"].isna(),"RSQ"] = valid_r2
     else:
         if verbose: log.write(" -Lead SNP not found in reference...")
         sumstats["RSQ"]=None
@@ -1019,29 +1070,36 @@ def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_thresh
 
 # -
 
-""
-def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding):
+def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding,gtf_chr_dict,gtf_gene_name):
     #loading
-    if gtf_path =="defualt" or gtf_path =="ensembl":
-        gtf = get_gtf(chrom=str(region[0]),build=build,source="ensembl")
+    if gtf_path =="default" or gtf_path =="ensembl":
+        to_query_chrom = gtf_chr_dict[region[0]]
+        gtf = get_gtf(chrom=to_query_chrom,build=build,source="ensembl")
     elif gtf_path =="refseq":
-        gtf = get_gtf(chrom=str(region[0]),build=build,source="refseq")
+        gtf = get_gtf(chrom=to_query_chrom,build=build,source="refseq")
     else:
-        gtf = pd.read_csv(gtf_path,sep="\t",header=None)
-
+        gtf = pd.read_csv(gtf_path,sep="\t",header=None, comment="#",low_memory=False,dtype={0:"string"})
+        gtf = gtf.loc[gtf[0]==gtf_chr_dict[region[0]],:]
     #filter in region
-    genes_1mb = gtf.loc[(gtf[0]==str(region[0]))&(gtf[3]<region[2])&(gtf[4]>region[1]),:].copy()
+    #gtf_chr_dict
+    genes_1mb = gtf.loc[(gtf[0]==to_query_chrom)&(gtf[3]<region[2])&(gtf[4]>region[1]),:].copy()
     genes_1mb.loc[:,"gene_biotype"] = genes_1mb[8].str.extract(r'gene_biotype "([\w\.\_-]+)"')
-
-    if gtf_path=="refseq":
-        genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"')
+    if gtf_gene_name is None:
+        if gtf_path=="refseq":
+            genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"').astype("string")
+        elif gtf_path =="default" or gtf_path =="ensembl":
+            genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_name "([\w\.-]+)"').astype("string")
+        else:
+            genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"').astype("string")
     else:
-        genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_name "([\w\.-]+)"')
+        pattern = r'{} "([\w\.-]+)"'.format(gtf_gene_name)
+        genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(pattern).astype("string")
+
     genes_1mb.loc[:,"gene"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"')
+
     exons = genes_1mb.loc[genes_1mb[2]=="exon",:].copy()
     if region_protein_coding is True:
         genes_1mb  =  genes_1mb.loc[genes_1mb["gene_biotype"]=="protein_coding",:].copy()
-
     #uniq genes
     uniq_gene_region = genes_1mb.loc[genes_1mb[2]=="gene",:].copy()
     flank = region_flank_factor * (region[2] - region[1])
@@ -1089,7 +1147,6 @@ def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding)
     return uniq_gene_region, exons
 
 
-""
 def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50):
         gene = data.gene_names_at_locus(contig=x[chrom], position=x[pos])
         if len(gene)==0:
